#!/bin/bash
# Executes a database backup for the specified database container.
# The current directory is expected to contain a project configured
# as with SSDT conventions for an application database.
#
# When successful, the output file will be in ./backup with the
# container name and timestamp in the file.
# The file will also be placed on the specified remote target
# The format of the output is a compressed pg_dump (sql) format.
# along with the top level contents of the project directory
# Environment variables can be used for REMOTE_BACKUP_TARGET and REMOTE_USERNAME
#


container=${1?Must provide container name to backup}
remoteTarget=${2:-$REMOTE_BACKUP_TARGET}
userName=${3:-$REMOTE_USERNAME}
projectDir=${4:-$PWD}


cd $projectDir

source "${SSDT_SCRIPTS:-$(dirname "${BASH_SOURCE[0]}")}/.functions.sh"

set -o pipefail
mkdir -p ./backup

project=$(composeGetProject)

IRN=$(docker-compose exec -T $container psql --username=postgres --dbname=$container -t -c 'select irn from organization') 
##Trim function in postgres didn't work - so take out the extra space this way.
IRN=`echo $IRN | sed -e 's/^[[:space:]]*//'`

## if database is empty, it will put IRN 000000
if [ "$IRN" == "" ]; then
   echo "no IRN set, using 000000"
    IRN=000000
fi

echo "Project is $project"
echo "Container is $container"
echo "Remote target is $remoteTarget"
echo "Username is $userName"
echo "IRN is $IRN"

backupFile=./backup/${IRN}${project}-${container}.$(date +%Y-%m-%d-%H-%M-%S).backup
backupFile2=./backup/${IRN}${project}-${container}.$(date +%Y-%m-%d-%H-%M-%S).directorycontents.tar.gz

#If the project is empty, we want to stop the process because this is being run from the wrong directory
if [ "$project" == "" ]; then
   echo "no project available"
   exit 1
fi

echo "starting backup of $container for $project"
docker-compose exec -T $container sh -c "gosu postgres pg_dump -Cc --if-exists --dbname=$container ; (exit $?) " > ${backupFile}

if [[ $( grep --count "CREATE TABLE" ${backupFile} ) -lt 200 || $( grep --count "PostgreSQL database dump complete" ${backupFile} ) -eq 0 ]]; then
   echo "ERROR: backup verification FAILED"
   echo "Error: empty database"
   echo "ERROR:  $(tail ${backupFile})"
  # We want the process to continue even if the db is blank
  # exit 1
fi

gzip ${backupFile}

echo "completed backup of $container for $project to ${backupFile}"

#backup of all files in current directory
#tar -czf ${backupFile2} . --exclude=./backup
tar --exclude=./backup -czf ${backupFile2} .

echo "completed backup of all files for $project to ${backupFile2}"

#Create remote directory before scp

host=`echo $userName@$remoteTarget | sed -e 's/:.*//'`

target=`echo $remoteTarget | sed -e 's/.*://'`

ssh $host mkdir -p $target/$project

#Create remote directory
#
scp ${backupFile}.gz  ${backupFile2} $userName@$remoteTarget/$project

echo " "

echo "completed sending ${backupFile}.gz and ${backupFile2} to ${remoteTarget}/${project} as user $userName"


